;--------------------------------------------------------------------------------
; ssbeacon.asm
; Homing Beacon for the Super Scooper "vacuum" robot
; Written for 12C508 PIC microcontroller
; Version 1.0
; by Dale A. Heatherington March 7 2001
;
;
; The IR output carrier frequency is 40 khz.
; The ultrasonic carrier is also 40khz.
; The IR received carrier is 33 khz.
;
; Operation summary:
; At the trailing edge of the incoming 33khz IR pulse from the robot, this
; code emits a 1000us 40 khz IR pulse on all 3 LEDS to set the receivers AGC.
; Next it emits a 1000us 40khz pulse on IR LED #1 then another on LED #2  and finally LED #3.
; And lastly, it emits a 500us 40khz  pulse on the ultrasonic transmitter.
; Since all three IR LEDs are pointed in
; different directions the robot can tell which LED beam it's in
; by noting which time slot it received a reply.  Also, the time delay
; of the returning ultrasonic pulse tells the robot how far away the
; beacon is.
;
;
;
;---------------------------------------------------------------------
;
;   PIC12C508 pin assignments
;
; 1     +5v
; 2     ultrasonic transmitter
; 3     Visible indicator LED
; 4     IR receiver input (Low = pulse received)
; 5     LED 3 drive and piezo transducer (low true)
; 6     LED 2 drive
; 7     LED 1 drive
; 8     GND
;-----------------------------------------------------------------------
;
;;----------------------------------------------------------------------
;;This program is free software; you can redistribute it and/or modify  
;;it under the terms of the GNU General Public License as published by      
;;the Free Software Foundation; either version 2 of the License, or         
;;(at your option) any later version.                                       
;;
;;This program is distributed in the hope that it will be useful,       
;;but WITHOUT ANY WARRANTY; without even the implied warranty of            
;;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
;;GNU General Public License for more details.                              
                                                                          
;;You should have received a copy of the GNU General Public License     
;;along with this program; if not, write to the Free Software               
;;Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
;;------------------------------------------------------------------------    


	RADIX	        dec
	processor	12c508


	__CONFIG h'0FEA'		;Internal 4 RC mhz osc
        

                

        include "p12c508.inc"




 
;------------------------------------------------------------

        
;GPIO  input bits
IR1     equ     3      ;IR sensor input (low true, pin 4)

;GPIO output bits

LED_1    equ     0      ;LED 1
LED_2    equ     1      ;LED 2  IR LED driver outputs (low true)
LED_3    equ     2      ;LED 3

RED_LED equ     4       ;Red indicator LED out (low true)
PZ      equ     5       ;piezo ultrasonic transmitter output

gp_save equ     07h
gp_last equ     08h
count   equ     09h
tdelay  equ     0ah
duration equ    0bh     ;indicates how long IR pulse was




;---------------------------------------------------------
;Wait N * 5uS
delay   macro   N
        local   _loop
        movlw   N
        movwf   count
_loop   nop
        nop
        decfsz  count,f
        goto    _loop
        endm
        
;One cycle of 40 KHZ (less 3uS for overhead in other code)
;This takes 22uS to execute. A complete cycle is 25 uS long.
;
cycle_led      MACRO  _LED

        bcf     GPIO,_LED
        nop
        nop
        nop
        nop
	nop
	nop
	nop
	nop
        nop
        nop
        nop
        bsf     GPIO,_LED
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop

        endm


;Cycle all the LEDS at once
cycle_all_led      MACRO

        bcf     GPIO,LED_1
        bcf     GPIO,LED_2
        bcf     GPIO,LED_3
        nop
        nop
	nop
	nop
	nop
	nop
        nop
        nop
        nop
        bsf     GPIO,LED_1
        bsf     GPIO,LED_2
        bsf     GPIO,LED_3
        nop
        nop
        nop
        nop
        nop
        nop
        nop

        endm


 ;Cycle piezo ultrasonic transmitter

cycle_pz      MACRO

        nop
        bcf     GPIO,PZ
        nop
        nop
        nop
	nop
	nop
	nop
	nop
        nop
        nop
        nop
        nop
        bsf     GPIO,PZ
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop

        endm



;code starts here...



	ORG 0
 
main_code
        movwf   OSCCAL      ;Internal 4 mhz osc calibration value
	clrf    STATUS
	movlw   0ffh
	movwf   GPIO
        movlw   08h         ;Bit 3 is the IR sensor input
	tris    GPIO        ;
        movlw   0C0h
	option
        clrf    duration  

wait_trigger
        comf    GPIO,W          ; w = ~GPIO
        movwf   gp_save         ; gp_save = w
        andlw   08h             ;Keep only IR receiver bit
        btfsc   STATUS,Z
        goto    IR_false        ;It's zero, could be a trailing edge.
        movwf   gp_last         ;save current value in gp_last
        incfsz  duration,f      ;count how long IR pulse lasts  (9 uS intervals) 
        goto    wait_trigger
        decf    duration,f      ;hold duration at 255  max
        goto    wait_trigger
        
IR_false

        movlw   08h 
        andwf   gp_last,w
        btfss   STATUS,Z        ;See if it was false last time too
        goto    trailing_edge   ;If not then we have a trailing edge
        movf    gp_save,w       ;..else save the current condition
        movwf   gp_last         ;..in gp_last
        clrf    duration        ;set duration counter to zero
        goto    wait_trigger    ;and go back to wait 
        

trailing_edge

        clrf    gp_last
        clrf    gp_save
        
       
        movlw   89
        addwf   duration,W
        btfsc   STATUS,C        ;check for carry. If carry, pulse was longer than 1.5ke ms
        goto    skipit          ;do not respond if pulse >= 1.5 ms
        clrf    gp_last         ; clear gp_last
;
;                               ; Now we are at the trailing edge of the received pulse
;


        bcf     GPIO,RED_LED    ;Flash the red indicator LED
;
;
        movlw   40
        movwf   count
pulseall
        cycle_all_led           ;Pulse all LEDs for 1 ms to set the AGC on the
        decfsz  count,f         ;robots IR receivers
        goto    pulseall


        movlw   40
        movwf   count
pulse1  cycle_led led_1         ;Make the 1st 1000uS beacon reply pulse (IR LED #1)
        decfsz  count,f
        goto    pulse1

        movlw   40
        movwf   count
pulse2  cycle_led led_2         ;Make the 2nd 1000uS beacon reply pulse (IR LED #2)
        decfsz  count,f
        goto    pulse2

        movlw   40
        movwf   count
pulse3  cycle_led led_3           ;Make the 3rd 1000uS beacon reply pulse (Both IR LED #3 and ultrasonic)
        decfsz  count,f
        goto    pulse3

pulse_sonar
        movlw   20              ;Make 500us 40khz sonar pulse
        movwf   count
psloop  cycle_pz
        decfsz  count,f
        goto    psloop


        delay   100              ;wait 500us
        bsf     GPIO,RED_LED     ;Indicator LED off
skipit  clrf    duration
        goto    wait_trigger
        
    
        end
        
    
        
            
            
        
        
        
        
